import numpy as np
import pandas as pd
import stan
import matplotlib as mplt
import matplotlib.pyplot as plt
import seaborn as sb
import arviz as az

finnm = input('Input Data File (*.xlsx) = ')

xlsx_fl = pd.ExcelFile(finnm)
data_xlsx = pd.read_excel(xlsx_fl,  header = None)
data = data_xlsx.values

foutnm = input('Output Text File (*.txt) = ')
fout = open(foutnm, 'w')
fout.write('Input Data File = {}\n'.format(finnm))

N = len(data) - 1
Sts = [v for v in data[1:, 1]]
MEs = [v for v in data[1:, 2]]
print('N =', N)
print(len(Sts))
print(len(MEs))

fout.write('\nData...\n')
for st, me in zip(Sts, MEs):
    print(st, me)
    fout.write(f'{st:>10}{me:>10}\n')

Data = {'N':N, 'Sts':Sts, 'MEs':MEs}

with open('MEc.stan', 'r') as fstan:
    sm = stan.build(fstan.read(), data = Data)
fit = sm.sample(num_samples = 5000) 

print(fit)

i_data = az.from_pystan(posterior = fit, posterior_model = sm)
smry = az.summary(i_data, var_names = ['a', 'b', 'c', 'sgm'])  
print('Summary =\n',smry)
fout.write(f'\nSummary:\n{smry}\n')

az.plot_trace(i_data, var_names = ['a', 'b', 'c'])
plt.tight_layout()
plt.show()

d_fm = fit.to_frame()

fig = plt.figure(figsize = (7,7))
ax = fig.add_subplot(111, projection = '3d')
ax.scatter(d_fm['a'], d_fm['b'], d_fm['c'])
ax.set_xlabel('a', fontsize = 16)
ax.set_ylabel('b', fontsize = 16)
ax.set_zlabel('c', fontsize = 16)
plt.title('Scatter plot of (a,b,c)', fontsize = 20)
plt.show()

mean_a = d_fm['a'].mean() 
P025_a, P50_a, P975_a = np.percentile(fit['a'], [2.5, 50, 97.5])
plt.title('Posterior Distribution of a', fontsize = 20)
sb.kdeplot(d_fm['a'], clip = (0.0, np.max(d_fm['a'])),
           label ='Mean = {0:.2f}\nMed = {1:.2f}\n95%CI = [{2:.2f}, {3:.2f}]'.
           format(mean_a, P50_a, P025_a, P975_a))
plt.xlabel('a', fontsize = 18)
plt.ylabel('Density', fontsize = 14)
plt.legend(fontsize = 12)
plt.tight_layout()
plt.show()

mean_b = d_fm['b'].mean()  
P025_b, P50_b, P975_b = np.percentile(fit['b'], [2.5, 50, 97.5])
plt.title('Posterior Distribution of b', fontsize = 20)
sb.kdeplot(d_fm['b'],
           label = 'Mean = {0:.2f}\nMed = {1:.2f}\n95%CI = [{2:.2f}, {3:.2f}]'.
           format(mean_b, P50_b, P025_b, P975_b))
plt.xlabel('b', fontsize = 18)
plt.ylabel('Density', fontsize = 14)
plt.legend(fontsize = 12)
plt.tight_layout()
plt.show()

mean_c = d_fm['c'].mean()  
P025_c, P50_c, P975_c = np.percentile(fit['c'], [2.5, 50, 97.5])

plt.title('Posterior Distribution of c', fontsize = 20)
sb.kdeplot(d_fm['c'],
           label ='Mean = {0:.2f}\nMed = {1:.2f}\n95%CI = [{2:.2f}, {3:.2f}]'.
           format(mean_c, P50_c, P025_c, P975_c))
plt.xlabel('c', fontsize = 18)
plt.ylabel('Density', fontsize = 14)
plt.legend(fontsize = 12)
plt.tight_layout()
plt.show()

min_St = min(Sts)
max_St = max(Sts)
St_Values = np.arange(min_St, max_St, (max_St - min_St) / 1000.0)

def PowerLaw(x, a, b, c):
    return a * ((x - c) ** b)

St_grid = np.linspace(min_St, max_St, 100)
tn_samples = len(d_fm)
n_samples = 500
idx_samples = np.random.choice(range(tn_samples), n_samples, replace = False)
curves_smpl = []
for s in idx_samples:
    va = d_fm['a'][s]
    vb = d_fm['b'][s]
    vc = d_fm['c'][s]
    phis = PowerLaw(St_grid, va, vb, vc)
    plt.plot(St_grid, phis, ls = ':', lw = 1, alpha = 0.5)
    curves_smpl.append(phis)
curves_smpl = np.array(curves_smpl)
c_s_q1, c_s_q3 = np.percentile(curves_smpl, [25, 75], axis = 0)
plt.plot(St_grid, c_s_q1, c = 'b', ls = '--', label = 'Q1')
plt.plot(St_grid, c_s_q3, c = 'g', ls = '--', label = 'Q3')
plt.plot(St_Values, [PowerLaw(v, P50_a, P50_b, P50_c) for v in St_Values],
         'r-', label = r'$\psi=a(\varphi-c)^b$' + ('\na = {0:.3f}'.format\
                        (P50_a)) + ('\nb = {0:.3f}'.format(P50_b)) +
                      ('\nc = {0:.3f}'.format(P50_c)))
plt.plot(Sts, MEs, 'o', c = 'b', alpha = 0.5, label = 'Data')
plt.legend()
plt.show()
    
print()
print(foutnm + ' was saved.')
print()
